/************************************************************************************
gpsserviceirs
Copyright (c) 2010, International Rescue System Institute
All rights reserved.

Contact us if you use this software for sell.
If you use this software not for sell, you can use this software under BSD license.
See the files LICENSE.TXT and LICENSE-BSD.TXT for more details.                     
************************************************************************************/

//***********************************************************************************************//
//* legacy.cpp											*//
//***********************************************************************************************//


//-- [ヘッダファイル] ---------------------------------------------------------------------------//
#include "legacy.h"											// legacyヘッダファイル読み込み
//-----------------------------------------------------------------------------------------------//


//-- [シグナル処理関数] -------------------------------------------------------------------------//
//- void signal_handler_IO(int status)								-//
//- 機能  ：RS-232Cからの受信完了時フラグ変更							-//
//- 引数  ：(int)  シグナル									-//
//- 戻り値：(void) なし										-//
//-----------------------------------------------------------------------------------------------//
void signal_handler_IO(int status)
{
	wait_flag = 0;											// フラグリセット

	return;												// 戻り値

}
//-----------------------------------------------------------------------------------------------//


//-- [シリアルポート初期化関数] -----------------------------------------------------------------//
//- void rs232c_init(int *fd, struct sigaction *saio, struct termios *oldtio			-//
//-			struct termios *newtio)							-//
//- 機能  ：シリアルポートの初期化を行う							-//
//- 引数  ：(int *)		 ファイルディスクリプタポインタ					-//
//-	  ：(struct sigaction *) シグナルハンドル設定格納構造体ポインタ				-//
//-	  ：(struct termios *)   初期ポート設定格納構造体ポインタ				-//
//-	  ：(struct termios *)   新規ポート設定格納構造体ポインタ				-//
//-	  ：(char *)		 COMアドレス格納配列ポインタ					-//
//-	  ：(unsigned long int)  c_cflag用引数設定値						-//
//-				 [ボーレート・ストップビット・ハードシェイク]			-//
//- 戻り値：(void)		 なし								-//
//-----------------------------------------------------------------------------------------------//
void rs232c_init(int *fd, struct sigaction *saio, struct termios *oldtio, struct termios *newtio,
	char *modemdevice, unsigned long int num_cflag)
{

	// シリアルポートオープン
	if((*fd = open(modemdevice, O_RDWR | O_NOCTTY | O_NONBLOCK)) < 0){				// シリアルポートオープン
		perror(modemdevice);									// システムエラーメッセージ出力
		exit(-1);										// 強制終了
	}

	// シグナルハンドル設定
	saio->sa_handler = signal_handler_IO;								// シグナルハンドラ
	sigemptyset(&saio->sa_mask);									// 追加マスクシグナル
	saio->sa_flags = 0;										// 動作詳細設定フラグ
	saio->sa_restorer = NULL;									// オブソリート [= NULL]

	// シグナルアクションの調査・変更
	sigaction(SIGIO, saio, NULL);									// シグナルアクション調査・変更

	// 非同期通信モード設定
	fcntl(*fd, F_SETOWN, getpid());									// シグナル受け取り設定 [プロセス]
	fcntl(*fd, F_SETFL, FASYNC);									// 非同期通信モード設定

	// 前処理
	tcgetattr(*fd, oldtio);										// 初期ポート設定退避

	// シリアルポート設定
	bzero(newtio, sizeof(*newtio));									// 新規ポート設定初期化
	newtio->c_cflag = (CS8 | CLOCAL | CREAD | num_cflag);						// 新規ポート設定
	newtio->c_iflag = IGNPAR;									// 新規ポート設定
	newtio->c_oflag = 0;										// 新規ポート設定
	newtio->c_lflag = 0;										// 新規ポート設定

	// 読み込み設定
	newtio->c_cc[VMIN] = 1;										// ブロック間文字数 [1byte]
	newtio->c_cc[VTIME] = 0;									// タイマ未使用

	// データ処理
	tcflush(*fd, TCIFLUSH);										// データ破棄

	// 新規ポート設定有効
	tcsetattr(*fd, TCSANOW, newtio);								// 設定反映

	return;												// 戻り値

}
//-----------------------------------------------------------------------------------------------//


//-- [シリアルポート終了処理関数] ---------------------------------------------------------------//
//- void rs232c_final(int *fd, struct termios *oldtio)						-//
//- 機能  ：シリアルポートの終了処理を行う							-//
//- 引数  ：(int *)		 ファイルディスクリプタポインタ					-//
//-	  ：(struct termios *)   初期ポート設定格納構造体ポインタ				-//
//- 戻り値：(void)		 なし								-//
//-----------------------------------------------------------------------------------------------//
void rs232c_final(int *fd, struct termios *oldtio)
{

	// 後処理
	tcsetattr(*fd, TCSANOW, oldtio);								// 初期ポート設定復帰

	// シリアルポートクローズ
	close(*fd);											// シリアルポートクローズ

	return;												// 戻り値

}
//-----------------------------------------------------------------------------------------------//


//-- [時間確認関数] -----------------------------------------------------------------------------//
//- double gettimeofday_sec(void)								-//
//- 機能  ：現在時刻の取得を行う								-//
//- 引数  ：(void)   なし									-//
//- 戻り値：(double) 現在時刻 [s]								-//
//-----------------------------------------------------------------------------------------------//
double gettimeofday_sec(void)
{

	//-- [変数宣言] -------------------------------------------------------------------------//
	//-											-//
	//- struct time{									-//
	//-											-//
	//-	time_t tv_sec;			- 秒						-//
	//-	suseconds_t tv_usec;		- マイクロ秒					-//
	//-											-//
	//- }											-//
	//-											-//
	//---------------------------------------------------------------------------------------//
	struct timeval tv;										// 使用されたユーザ時間格納構造体

	gettimeofday(&tv, NULL);									// 時刻取得

	return tv.tv_sec + (double)tv.tv_usec * 1e-6;							// 戻り値 [現在時刻]

}
//----------------------------------------------------------------------------------------------//
